import os
import pylab
import numpy as np
from optparse import OptionParser

from dirsdist.histograms import mlab
import dirsdist.histograms as hg
import dirsdist.linalg as la
from dirsdist.utils import construct_name

# ******* PROGRAM OPTIONS *******

usage = """%prog [options] files file_out"""

help = {
    'surface' :
    'surface of histograms = counting + drawing  [default: %default]',
    'histogram':
    'histograms = counting + drawing  [default: %default]',
    'histogram_fraction':
    'histograms_fraction = counting + drawing  [default: %default]',
    'rose_orig':
    'original rose of directions = counting + drawing  [default: %default]',
    'rose_scaled':
    'original rose of directions = couting + drawing the rose of directions scaled by suma of the lengths of the objects in given directions  [default: %default]',
    'card_orient':
    'cardinal directions = counting + drawing  [default: %default]',
    'card_vectors':
    'n-max vectors of cardinal directions = counting + drawing [default: %default]',
    'one_file':
    'add vectors of cardinal directions and/or isotropy parameters on the end of the file [default: %default]',
    'n_max':
    'number of the cardinal directions  [default: %default]',
    'threshold':
    'threshold fraction of cardinal directions  [default: %default]',
    's' :
    "save all drawing pictures and text files with angles of main objects' directions",
    'prefix' :
    "prefix of output files  [default: %default]",
    'format' :
    "format of ouput files  [default: %default]",
    'skip_rows' :
    "number of rows to skip in histogram files [default: %default]",
    'isotropy' :
    "evaluate the isotropy of the system [default: %default]",
    'calibration' :
    "calibration of images used for fractional and ellipsoidal anisotropy evaluation [default: %default]",
    'anisotropy' :
    "count fractional and ellipsoidal anisotropy of line system [default: %default]",
}

def main():
        
    parser = OptionParser(usage=usage, version="%prog ")
    parser.add_option("", "--surface", 
                      action="store_true", dest="surface",
                      default=False, help=help['surface'])
    parser.add_option("", "--histogram", 
                      action="store_true", dest="histogram",
                      default=False, help=help['histogram'])
    parser.add_option("", "--histogram-fraction", 
                      action="store_true", dest="histogram_fraction",
                      default=False, help=help['histogram_fraction'])
    parser.add_option("", "--rose-orig", 
                      action="store_true", dest="rose_orig",
                      default=False, help=help['rose_orig'])
    parser.add_option("", "--rose-scaled", 
                      action="store_true", dest="rose_scaled",
                      default=False, help=help['rose_scaled'])
    parser.add_option("", "--card-orient", 
                      action="store_true", dest="card_orient",
                      default=False, help=help['card_orient'])
    parser.add_option("", "--card-vectors", 
                      action="store_true", dest="card_vectors",
                      default=False, help=help['card_vectors'])
    parser.add_option("", "--one-file", 
                      action="store_true", dest="one_file",
                      default=False, help=help['one_file'])
    parser.add_option("", "--n-max", type=int, metavar='int',
                      action="store", dest="n_max",
                      default=3, help=help['n_max'])
    parser.add_option("", "--threshold", type=float, metavar="float",
                      action="store", dest="threshold",
                      default=None, help=help['threshold'])
    parser.add_option("-s", "", 
                      action="store_true", dest="s",
                      default=False, help=help['s'])
    parser.add_option("", "--prefix", metavar='prefix', 
                      action="store", dest="prefix",
                      default=None, help=help['prefix'])
    parser.add_option("", "--format", metavar='format',
                      action="store", dest="format",
                      default="pdf", help=help['format'])
    parser.add_option("", "--skip-rows", type=int, metavar="int",
                      action="store", dest="skip_rows",
                      default=0, help=help['skip_rows'])
    parser.add_option("", "--isotropy", 
                      action="store_true", dest="isotropy",
                      default=False, help=help['isotropy'])                      
    parser.add_option("", "--calibration", metavar='float,float,float',
                      action="store", dest="calibration",
                      default='0.180879, 0.180879, 6', help=help['calibration'])
    parser.add_option("", "--anisotropy", 
                      action="store_true", dest="anisotropy",
                      default=False, help=help['anisotropy'])                      
                 
    options, args = parser.parse_args()
    
    options.calibration = [float(r) for r in options.calibration.split(',')]
        
    if options.prefix:
        prefix = options.prefix + '_'
    else: 
        prefix = ''
        
    
    print options
        
    # combination of meridians and parallels
    #color_vector = np.repeat( (1.0, 0.5, 1.0), (1.0, 1.0, 0.5), (0.5, 1.0, 1.0))

    # surface of histograms = names of output path; output name
    output_path_surf = 'surface'
    output_filename_surf = prefix + 'surface_%03d_%03d.' + options.format

    # histograms = names of output path; output name
    output_path_histograms = 'histograms'
    output_filename_histograms = prefix + 'histogram_mean_%03d_%03d.' \
                                                + options.format
    
    # histograms_fraction = names of output path; output name
    output_path_histograms_fraction = 'histograms_fraction'
    output_filename_histograms_fraction = prefix + 'histogram_mean_fraction_%03d_%03d.' \
                                                + options.format
    
    # vectors of n_max main directions  = elevations, azimuths = counting; 
    # names of output path; output name
    output_path_vector = 'vectors'
    output_filename_vector = prefix +  'vectors_%03d_%03d.txt'
    output_filename_vector_mean = prefix + 'mean_vectors_%03d_%03d.txt'

    # evaluate isotropy
    # names of output path; output name
    output_path_isotropy = 'isotropy'
    output_filename_isotropy = prefix +  'isotropy_%03d_%03d.txt'
        
    # original rose of directions = names of output path; output name
    output_path_rose = 'rose_orig'
    output_filename_rose = prefix + 'rose_orig_%03d_%03d.' \
                                     + options.format   

    # scaled rose of directions = names of output path; output name
    output_path_rose_scaled = 'rose_scaled'
    output_filename_rose_scaled = prefix + 'rose_scaled_%03d_%03d.' \
                                                 + options.format

    # cardinal directions = names of output path; output name
    output_path_orientation = 'cardinal_orientation'
    #output_filename_orientation = prefix + 'orientation.' + options.format
    output_filename_orientation = prefix + 'orientation.' + 'png'


# ****** PROGRAM ******* 

    
    # inicialization of data_sum
    data_sum = None
    
    for ii, iFile in enumerate(args):
            if options.anisotropy:
                n_tracer_data, tracer_data = hg.load_tracer_data(iFile)
            else:
                data = np.loadtxt(iFile, skiprows=options.skip_rows) 
                if data_sum is None:
                    data_sum = data
                else:
                    data_sum[:,2] += data[:,2]   
    
    if not options.anisotropy:
        data = data_sum
        data_sorted = hg.sort_data(data)
        fraction = hg.compute_fraction(data_sorted)
    
        vs = data[:,0].max() + 1
        hs = data[:,1].max() + 1
        print 'v_elev, h_azim: ', vs, hs
    
    if options.threshold is not None:
        print '___Threshold - evaluation of cardinal directions___'
        options.n_max = hg.find_n_max(fraction, options.threshold)
    
    if options.anisotropy:
        print '___Evaluation of fractional and ellipsoidal anisotropy___'
        
        ellipsoidal_anisotropy, fractional_anisotropy = hg.evaluate_anisotropy(n_tracer_data, 
                                                                            tracer_data, options.calibration)
    
    if options.isotropy:
        print '___Evaluation of isotropy___'
        
        iso = chi_square, p_val, mean_fraction, std_fraction = hg.evaluate_isotropy(data)
        iso = [(iso)]
        if options.s:
            name = construct_name(output_path_isotropy, 
                                                output_filename_isotropy % (vs, hs))
            if options.one_file:
                h = open(name, 'a')
                np.savetxt(h, iso)
                h.close
            else:
                h = open(name, 'w')
                np.savetxt(h, iso)
                h.close
        
        
    if options.surface:
        mlab.figure(1)
        mlab.surf( np.arange(hs), np.arange(vs),
                        data[:,2].reshape( (hs, vs) ), colormap='gist_earth')
        if options.s:
            name = construct_name(output_path_surf, 
                                                output_filename_surf % (vs, hs))
            mlab.savefig(name)
            mlab.show()
        else:
            mlab.show()

    if options.histogram:
        hg.histogram_mean(2, data, options.prefix)
        fig = pylab.gcf()
        if options.s:
            name = construct_name(output_path_histograms, 
                                                    output_filename_histograms % (vs, hs) )
            print name
            fig.savefig( name )
            pylab.show()
        else:
            pylab.show()
                
    
    if options.histogram_fraction:
        fraction_unsort = hg.compute_fraction(data)
        hg.histogram_mean_fraction(3, data, fraction_unsort, options.prefix)
        fig = pylab.gcf()
        if options.s:
            name = construct_name(output_path_histograms_fraction, 
                                                    output_filename_histograms_fraction % (vs, hs) )
            print name
            fig.savefig( name )
            pylab.show()
        else:
            pylab.show()
    
    if options.card_vectors:
        print """shape, vector, vector_means
        shape = (h_azim, v_elev),
        vector = [azim1, azim2, elev1, elev2], 
        vector_mean = [azim_mean, elev_mean]"""
        shape, vector, vector_mean = hg.angle_max(data_sorted, options.n_max)
        vector = np.r_[vector, fraction[None, -options.n_max:]]
        vector_mean = np.r_[vector_mean, fraction[None, -options.n_max:]]
        print shape, vector, vector_mean
        if options.s:
            name = construct_name(output_path_vector, 
                                                output_filename_vector % (vs, hs))
            if options.one_file:
                h = open(name, 'a')
                np.savetxt(h, vector)
                h.close
            else:
                h = open(name, 'w')
                np.savetxt(h, vector)
                h.close
        if options.s:
            name = construct_name(output_path_vector, 
                                                output_filename_vector_mean % (vs, hs))
            if options.one_file:
                h = open(name, 'a')
                np.savetxt(h, vector_mean)
                h.close
            else:
                h = open(name, 'w')
                np.savetxt(h, vector_mean)
                h.close
        
    if options.rose_orig:
        hg.rose(3, data)
        if options.s:
            name = construct_name(output_path_rose, 
                                                output_filename_rose % (vs, hs))
            mlab.savefig(name)
            mlab.show()
        else: 
            mlab.show()
      
    if options.rose_scaled:
        hg.rose_scaled(4, data)
        if options.s:
            name = construct_name(output_path_rose_scaled , 
                                                output_filename_rose_scaled % (vs, hs))   
            mlab.savefig(name)
            mlab.show()
        else: 
            mlab.show()
    
    if options.card_orient:
        print '___Cardinal directions___'
        v_card, h_card = hg.cardinal_orientation(5, data_sorted, ii, options.n_max,
                                                plot_axes = True)
        mlab.view(0, 90)
        if options.s:
            name = construct_name(output_path_orientation, 
                                                output_filename_orientation)
            mlab.savefig(name)
            mlab.show()
        else:
            mlab.show()    
        
        card_directions = la.convert_angles_to_direction(v_card, h_card)
        print 'card_directions'
        print card_directions
        
if __name__ == '__main__':
    main()
